@@ -8,9 +8,9 @@ use crate::{
 use nom::{
   branch::alt,
   bytes::complete::tag,
-  combinator::{map_res, value},
-  error::ParseError,
-  multi::fold_many1,
+  combinator::value,
+  error::{FromExternalError, ParseError},
+  multi::many1,
   sequence::tuple,
 };
 use std::{iter::Peekable, vec::IntoIter};
@@ -85,28 +85,33 @@ impl PrecedenceSolver {
 
     Ok(left)
   }
+
+  fn solve(tokens: IntoIter<Token>) -> Result<Expression, AstError> {
+    let mut solver = PrecedenceSolver {
+      token_iter: tokens.peekable(),
+    };
+    solver.expr(0)
+  }
 }
 
 pub(crate) fn binary_expr(i: LocatedSpan) -> IResult<Expression> {
   let (i, first_expr) = simple_expr(i)?;
-  let tokens = vec![Token::Expression(first_expr)];
-  map_res(
-    fold_many1(
-      tuple((binary_operator, simple_expr)),
-      tokens,
-      |mut tokens, (sep, expr)| {
-        tokens.push(Token::Operator(sep));
-        tokens.push(Token::Expression(expr));
-        tokens
-      },
-    ),
-    |tokens| {
-      let mut precedence_solver = PrecedenceSolver {
-        token_iter: tokens.into_iter().peekable(),
-      };
-      precedence_solver.expr(0)
-    },
-  )(i)
+  let first_token = Token::Expression(first_expr);
+  let (i, other_tokens) = many1(tuple((binary_operator, simple_expr)))(i)?;
+  let mut tokens = vec![first_token];
+  for (sep, expr) in other_tokens {
+    tokens.push(Token::Operator(sep));
+    tokens.push(Token::Expression(expr));
+  }
+
+  match PrecedenceSolver::solve(tokens.into_iter()) {
+    Ok(expr) => Ok((i, expr)),
+    Err(err) => Err(nom::Err::Error(nom::error::Error::from_external_error(
+      i,
+      nom::error::ErrorKind::MapRes,
+      err,
+    ))),
+  }
 }
 
 fn binary_operator(i: LocatedSpan) -> IResult<BinaryOperator> {
